function [Images, w, h, Cameras, imageList, cameraList, rawFileName] ...
    = voxelcolor_load_sequence(path, prefix, startNumber, cameraSuffix, diskMode, loadImages)
% Loads a sequence of images from disk
%
% input:
%   path         - directory with files
%   prefix       - file prefix to read
%   startNumber  - first image index to look for
%   cameraSuffix - will read a camera file with this extension together
%                  with an image, if found. Loading is done through the
%                  MatLab load command.
%   diskMode     - 0 means try to load in memory if possible, 1 means save
%                  as 'images.raw' immediately
%   loadImages   - 1 means try to load images; 0 will prevent loading of
%                  images but will do everything else (no 'images.raw' is 
%                  created). Should be used together with diskMode=1
%
% output:
%   Images      - Array of images
%   w, h        - width and height of images
%   Cameras     - Array with camera calibration parameters, per image
%                 (19 elements means Tsai model, 22 means Bouget model)
%   imageList   - list of image filenames
%   cameraList  - list of camera filenames
%   rawFileName - filename where the images are stored to disk to
%                 ('images.raw', but can be another name as well)
%
% Notes on camera files:
% - I use the .CPCC camera file extension, you may change it if you want
%   (cameraSuffix)
% - the Tsai camera model has a fixed 19 element file format (I have seen
%   it used in someones dataset anyway).
% - the 22 element Bouget format has been made by me; it is based on the
%   Bouget camera toolbox.

%   Tsai Camera Parameters (note: watch out for numbering problems)
%   ======================
%            00: Ncx
%            01: Nfx
%            02: dx
%            03: dy
%            04: dpx
%            05: dpy
%            06: Cx
%            07: Cy
%            08: sx
%            09: f
%            10: kappa1
%            11: Tx
%            12: Ty
%            13: Tz
%            14: Rx
%            15: Ry
%            16: Rz
%            17: p1 (not used?)
%            18: p2 (not used?)

%    Bouget Camera Parameters
%    ========================
%    fc1,fc2,cc1,cc2,alphac,kc1,kc2,kc3,kc4,kc5,
%    R11,R12,R13,R21,R22,R23,R31,R32,R33,T1,T2,T3

% by default, no images are stored to disk
rawFileName = '';

n = startNumber;
% look for existing files which start with image index 1
% numbers are allowed to have one leading 0, otherwise they aren't
% recognized
imageList = [];
cameraList = [];
while true
  filename = [prefix int2str(n)];
  filelist = dir([path '/' prefix int2str(n) '.*']);
  camlist  = dir([path '/' prefix int2str(n) '.' cameraSuffix]);
  if isempty(filelist) 
      filename = [prefix '0' int2str(n)];
      filelist = dir([path '/' prefix '0' int2str(n) '.*'])  ;
      camlist  = dir([path '/' prefix '0' int2str(n) '.' cameraSuffix]);
  end
  if isempty(filelist) 
      n = n - 1;
      break
  end
  
  if isempty(camlist) 
      n = n + 1;
      continue
  end
  
  % pick a file from the list (due to checking above this list cannot be
  % empty). If you have multiple versions, not always the same one might
  % be chosen!
  if(strcmp(filelist(1).name, [filename '.' cameraSuffix]))        
      fileToRead = [path '/' filelist(2).name];
  else
      fileToRead = [path '/' filelist(1).name];
  end

  imageList(size(imageList,2)+1).name = fileToRead;
  if isempty(camlist)
      cameraList(size(cameraList,2)+1).name = '';
  else
      cameraList(size(cameraList,2)+1).name = [path '/' camlist(1).name];
  end      
  n = n + 1;
end

imageCount = size(imageList, 2);

disp(['Found ' int2str(imageCount) ' sequence images in ' path ' with prefix ' prefix '.'])

Cameras = [];

% Initialize array.
for imageCounter = 1 : imageCount; 
    fileToRead = imageList(imageCounter).name;

    info = imfinfo(fileToRead);
    if imageCounter == 1       % Initialize the array?
        w = info.Width;
        h = info.Height;
        disp(['Image dimensions of ' int2str(w) 'x' int2str(h) '.']);
        try
            if diskMode > 0 
                error('fail');
            end
            Images = uint8(zeros([h w 3 imageCount]));
        catch
            disp('Using memory-efficient mode: images are transmitted through disk.');
            Images = uint32([w h 3 imageCount]);
            if loadImages
                rawFileName = tempname;
                fid = fopen(rawFileName,'w');
            end
        end
    end
    if ~((info.Width == w) && (info.Height == h))
        error(['invalid image dimensions in ' fileToRead]);       
    end
    if ~((info.BitDepth == 24) && (strcmp(info.ColorType, 'truecolor')))
        error(['invalid image color data in ' fileToRead]);
    end
    
    % do we have enough memory to read the images?
    if(size(size(Images),2) == 4)
        % possibly show images
        %imageview(double(Images(:,:,:,imageCounter - startNumber + 1))/256);
        image = imread(fileToRead);

        Images(:, :, :, imageCounter - startNumber + 1) = image;
    else
        if loadImages
            clear image;
            image = imread(fileToRead);
            % dump image to disk in RAW format, row by row
            for y = 1:h
                fwrite(fid,squeeze(image(y,:,:))','uint8');
            end
        end
    end       
    
    % read camera data (if cameraSuffix isn't empty), this is used for
    % loading Tsai CPCC files easily
    cameraFile = cameraList(imageCounter).name;
    if ~isempty(cameraFile)
        Cameras = [Cameras load(cameraFile)];    
    end
end
try
    fclose(fid);
catch
end